/*!
 * Koliseo KPerf Widget v1.0
 *
 * Copyright 2011 Nacho Coloma
 * Licensed under the MIT license.
 */

/**
  Displays a widget with a list of performances for a concrete event
  Requires: jquery
  Options:
    eventUuid {String} the event UUID (required)
    locale {String} the locale to use, default is the browser locale or "en" if none.
    weekStart {integer} first day of the week (0 is Sunday), default is locale specific default
    dateFormat {String} the date format to use, default is the locale-specific default.
    cssUrl {String} a CSS url to instead of the default from Koliseo.
*/

/**
  All locale-dependent entries
  You can specify new locales before loading this script. E.g.:
  window.Koliseo = {
    resources: {
      fr: {
        // entries go here
      }
    }
  };
*/
var allResources = $.extend({
  en: {
    ticketsAvailable: 'tickets available',
    seeOther: 'see other dates',
    buy: 'Get tickets',
    empty: 'There are no performances scheduled for this event.',
    dateFormat: 'Ddd Mm* d* h*:mm a.m',
    weekStart: 0,
    weekDays: 'Sunday Monday Tuesday Wednesday Thursday Friday Saturday',
    months: 'January February March April May June July August September October November December'
  },
  es: {
    ticketsAvailable: 'de entradas disponibles',
    seeOther: 'ver otras fechas',
    buy: 'Comprar',
    empty: 'No hay actuaciones programadas para este evento.',
    dateFormat: 'Ddd d* de Mm* HH:mm',
    weekStart: 1,
    weekDays: 'Domingo Lunes Martes Miércoles Jueves Viernes Sábado',
    months: 'Enero Febrero Marzo Abril Mayo Junio Julio Agosto Septiembre Octubre Noviembre Diciembre'
  }
}, window.Koliseo? window.Koliseo.resources : undefined);

// A unique widget ID
var widgetCount = 0
, cssInserted = false
;

$.fn.kperf = function(opt) {
  var 

    /** The unique ID of this widget */
    widgetId = widgetCount++,

    /** the options provided by the user */
    options,

    /** i18n resources for the current locale */
    res,

    /** the widget container */
    $element = this,

    /* the event uuid provided by the user */
    eventUuid,

    /** Transform 0-1 into 0-100 */
    toPercentage = function(value) {
      return +((value * 100).toFixed(0));
    },

    /** format a date to the current locale */
    formatDate = function (format, dateTimeStr) {
      var 
        date = new Date(dateTimeStr),
        parts = /(\d+)-(\d+)-(\d+)T(\d+):(\d+)/.exec(dateTimeStr),
        year = parts[1],
        month = parts[2],
        day = parts[3],
        hour = parts[4],
        hour12 = String(Number(hour) % 12 ? Number(hour) % 12 : 12),
        am = Number(hour) < 12 ? true : false, 
        minute = parts[5],
        nm = res.months.split(' ')[month - 1],
        nd = res.weekDays.split(' ')[date.getUTCDay()],
        padLeft = function(str) {
          str = String(str);
          var pad = "00";
          return pad.substring(0, pad.length - str.length) + str;
        };

      return format
        .replace(/yyyy/g, year)
        .replace(/yy/g, year.substr(2,2))
        .replace(/MMM/g, nm.substr(0,3).toUpperCase())
        .replace(/Mmm/g, nm.substr(0,3))
        .replace(/MM\*/g, nm.toUpperCase())
        .replace(/Mm\*/g, nm)
        .replace(/mM/g, padLeft(month))
        .replace(/DDD/g, nd.substr(0,3).toUpperCase())
        .replace(/Ddd/g, nd.substr(0,3))
        .replace(/DD\*/g, nd.toUpperCase())
        .replace(/Dd\*/g, nd)
        .replace(/dd/g, padLeft(day))
        .replace(/d\*/g, day)
        .replace(/HH/g, padLeft(hour))
        .replace(/H\*/g, hour)
        .replace(/hh/g, padLeft(hour12))
        .replace(/h\*/g, hour12)
        .replace(/mm/g, padLeft(minute))
        .replace(/AM/g, am ? "AM" : "PM")
        .replace(/am/g, am ? "am" : "pm")
        .replace(/A\.M/g, am ? "A.M." : "P.M.")
        .replace(/a\.m/g, am ? "a.m." : "p.m.")
        ;
    },

    /** Render the "buy" button */
    renderBuyButton = function($el) {
      $el.append('<p><a class="kbtn kprimary" href="' + getEventURL(eventUuid) + '/performances">' + res.buy + '</a></p>');
    },
    
    /** Render the provided list of performances */
    renderPerformances = function(performances) {
      var 
        html = '',
        $kperf = $element.find('.kperformances').length ?
                  $element.find('.kperformances') :
                  $('<div class="kperformances"></div>').appendTo($element)
      ;

      if (performances) {
        for (var i = 0; i < performances.length; i++) {
          var 
            performance = performances[i],
            availableTickets = performance.totalTickets - performance.soldTickets,
            pct = toPercentage(availableTickets / performance.totalTickets),
            ticketsClass = pct === 0? 'kticket-soldout' :
              pct < 20? 'kticket-danger' :
              pct < 40? 'kticket-warning' :
              'kticket-ok'
            ;
          html += 
            '<p class="kperformance" title="' + pct + '% ' + res.ticketsAvailable + '">' +
                '<a class="kperflink ' + ticketsClass + (pct === 0? ' disabled' : '') + '" href="' + getEventURL(performance.event.uuid) + '/performances?selected=' + performance.id + '" target="_blank">' + 
                  '<b class="kperfdate">' + formatDate(options.dateFormat, performance.dateTimeStr) + '</b> ' +
                  availableTickets + '/' + performance.totalTickets  + '<span class="kicon-ticket"></span> ' +
                  ' <span class="kprice">' + performance.minPrice + ' ' + performance.currency + '</span> ' + 
                '</a>' +
                '<span class="kaddr">' + performance.address + '</span>' +
            '</p>'
            ;

        }
      } else {
        html = res.empty;
      }

      $kperf.html(html);
    },

    getEventURL = function(eventUuid) {
      return (isOldIE() ? document.location.protocol : 'https:') + '//' + options.hostname + '/' + eventUuid;
    },

    /** insert kperf.css if not already present */
    insertStylesheet = function(url) {
      if (cssInserted) return;

      if (!url) {
        url = $('script[src*="/kperf"]').attr('src');
      }

      if (url) {
        url = url.replace('.js', '.css');
        document.createStyleSheet ?
          document.createStyleSheet(url) :
          $('head').append('<link rel="stylesheet" href="' + url + '">');
        cssInserted = true;
      }
    },

    /* error handler */
    ajaxError = function(jqXHR, errorThrown) {
      if ($element.hasClass('kloading')) {
        $element.removeClass('kloading');
        window.console && console.log(errorThrown);
        $element.addClass('kunsupported');
        renderBuyButton($element);
      }
    },

    /* Detect IE 8 and 9*/
    isOldIE = function() {
      return navigator.userAgent.match(/.+MSIE [89]\.0.+/);
    }
    ;

  if (!this.length)
    return;

  // initialize options with default values
  options = $.extend({
    hostname: 'www.koliseo.com',
    maxEntries: 5
  }, this.data(), opt);

  if (!options.eventUuid)
    throw new Error("options.eventUuid is required");

  eventUuid = options.eventUuid;

  // initialize CSS
  insertStylesheet(options.cssUrl);

  // initialize locale resources
  var locale = options.locale;
  if (!locale) {
    var parts = /([^\-]{2})-.+/.exec(navigator.language || navigator.userLanguage || '');
    parts && (locale = parts[1]);
  }
  res = options.res || allResources[locale] || allResources.en;
  options.dateFormat = options.dateFormat || res.dateFormat;


  // get performances and render
  $element.addClass('kcontainer').html('');
  if ($.support.cors) {
    $element.addClass('kloading');

    $.ajax({
      url: getEventURL(eventUuid) + '/performances',
      data: {
        minDate: new Date().valueOf()
      },
      dataType: 'json',
      type: 'GET',
      statusCode: {
        400: ajaxError,
        404: ajaxError
      },
      error: ajaxError,

      success: function(cursor) {
        $element.removeClass('kloading');
        
        $element.append($('<div class="kcalendar"></div>'));
        var calendarView = (new CalendarView({
          $el: $element.find('.kcalendar'),
          data: cursor.data,
          locale: res
        })).render()
        , selectDate = function($this) {
          calendarView.$el.find('.kcurrent').removeClass('kcurrent');
          $this.addClass('kcurrent');
          renderPerformances($this.data('performances'));
        };

        calendarView.$el.on('click keydown', '.kperfday', function(e) {
          if (e.keyCode && e.keyCode !== 13) return;
          selectDate($(this));
        });
        selectDate(calendarView.$el.find('.kperfday').first());
      }
    });
  } else {
    renderBuyButton($element);
  }

};

// initialize any .kcontainer element
$('.kcontainer').each(function(index, element) {
  $(element).kperf();
});
